package com.isu.gaswellwatch.modbus.data.decode.listener;

import cn.hutool.core.map.MapUtil;
import com.isu.gaswellwatch.modbus.data.ModbusMessage;
import com.isu.gaswellwatch.modbus.data.decode.DecodeHandler;
import com.serotonin.modbus4j.code.FunctionCode;
import com.serotonin.modbus4j.msg.ModbusResponse;
import com.serotonin.modbus4j.msg.ReadInputRegistersResponse;
import com.serotonin.modbus4j.serial.rtu.RtuMessageParser;
import com.serotonin.modbus4j.serial.rtu.RtuMessageResponse;
import com.serotonin.modbus4j.sero.util.queue.ByteQueue;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.amqp.core.BatchMessageListener;
import org.springframework.amqp.core.Message;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.jdbc.core.JdbcTemplate;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/11/23 0:43
 */
@Slf4j
@RequiredArgsConstructor
public class ModbusMessagePersistListener implements BatchMessageListener {

    private static final String COMMAND_SQL = "select * from commands where id = ?";

    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource(name = "redisTemplate")
    private RedisTemplate redisTemplate;
    @Resource
    private Map<String, DecodeHandler> decodeHandlerMap;

    @Override
    public void onMessage(Message message) {
        this.onMessageBatch(List.of(message));
    }

    @Override
    public void onMessageBatch(List<Message> messages) {

        try {
            Long commandId;
            String[] messageSplit;
            Map<String, Object> commandMap;
            String messageString, collectionMessage;
            Map<String, ModbusMessage.MessagePoint> messagePointMap;
//      网关标识/设备标识/命令标识/采集下发时间/采集数据接收时间/采集数据体
//      4B454E454E4731343030303030333538/1/10000/1732365690000/1732365691000/01042C07D000010003000C002B0029000300010000001A0018003303CD002300080DAC0DAC0000000000000959091DF066
            for (Message message : messages) {
                messageString = new String(message.getBody());
                messageSplit = StringUtils.split(messageString, "/");
                if (messageSplit.length < 6) {
                    log.error("非法数据: {}", messageString);
                    continue;
                }

                collectionMessage = messageSplit[5];
                commandId = NumberUtils.createLong(messageSplit[2]);
                commandMap = this.jdbcTemplate.queryForMap(COMMAND_SQL, commandId);
                if (ObjectUtils.isNotEmpty(commandMap)) {
                    log.error("指令[{}]不存在，数据: {}", commandId, messageString);
                    continue;
                }
                int index = MapUtil.getInt(commandMap, "start_address"), stepSize = 0;

                messagePointMap = new HashMap<>();
                ByteQueue byteQueue = new ByteQueue(collectionMessage);
                RtuMessageParser masterParser = new RtuMessageParser(true);
                RtuMessageResponse response = (RtuMessageResponse) masterParser.parseMessage(byteQueue);
                ModbusResponse modbusResponse = response.getModbusResponse();
                switch (modbusResponse.getFunctionCode()) {
                    case FunctionCode.READ_INPUT_REGISTERS: {
                        ReadInputRegistersResponse readInputRegistersResponse = (ReadInputRegistersResponse) modbusResponse;
                        for (short value : readInputRegistersResponse.getShortData()) {
                            stepSize = index * 4;
                            messagePointMap.put(StringUtils.leftPad(String.valueOf(index), 4, '0'),
                                    ModbusMessage.MessagePoint.builder()
                                            .parseValue(value)
                                            .originalValue(StringUtils.substring(collectionMessage, 6 + stepSize, 10 + stepSize))
                                            .build());
                            index++;
                        }
                    }
//                    case FunctionCode.READ_COILS: {
//                        ReadCoilsResponse readCoilsResponse = (ReadCoilsResponse) modbusResponse;
//                    }
//                    case FunctionCode.READ_DISCRETE_INPUTS: {
//                        ReadDiscreteInputsResponse readDiscreteInputsResponse = (ReadDiscreteInputsResponse) modbusResponse;
//                    }
//                    case FunctionCode.READ_HOLDING_REGISTERS: {
//                        ReadHoldingRegistersResponse readHoldingRegistersResponse = (ReadHoldingRegistersResponse) modbusResponse;
//                    }
//                    case FunctionCode.READ_EXCEPTION_STATUS: {
//                        ReadExceptionStatusResponse readExceptionStatusResponse = (ReadExceptionStatusResponse) modbusResponse;
//                    }
//                    case FunctionCode.REPORT_SLAVE_ID: {
//                        ReportSlaveIdResponse reportSlaveIdResponse = (ReportSlaveIdResponse) modbusResponse;
//                    }
                    default: {
                        log.error("FunctionCode {}, Message {}", modbusResponse.getFunctionCode(), collectionMessage);
                    }
                }


                Optional.ofNullable(commandMap.get("decode_handler_name"))
                        .map(this.decodeHandlerMap::get)
                        .ifPresent(decodeHandler -> decodeHandler.decode(ModbusMessage.builder()
                                .queueName(message.getMessageProperties().getConsumerQueue())
                                .commandId(commandId)
                                .message(collectionMessage)
                                .gatewayIdentifier(messageSplit[0])
                                .deviceId(NumberUtils.createLong(messageSplit[1]))
                                .collectionTime(NumberUtils.createLong(messageSplit[3]))
                                .receiveTime(NumberUtils.createLong(messageSplit[4]))
                                .messagePointMap(messagePointMap)
                                .build()));


            }
        } catch (Exception e) {
            throw new RuntimeException(e);

        }
    }
}
